#include "BasicQueries.h"

#include "../../datatype/AbstractData.h"
#include "../query/And.h"
#include "../query/DataSetName.h"
#include "../query/Distinct.h"
#include "../query/EqualTo.h"
#include "../query/Field.h"
#include "../query/Fields.h"
#include "../query/From.h"
#include "../query/Literal.h"
#include "../query/PropertyName.h"
#include "../query/Select.h"
#include "../query/Where.h"
#include "Utils.h"


std::unique_ptr<te::da::Select> te::da::BasicQueries::GetQuery(bool distinct, const std::vector<std::string>& vecColumns, const std::string& tableName, const std::vector<std::string>& vecWhereColumns, const std::vector<std::unique_ptr<te::dt::AbstractData>>& vecWhereValues)
{
  te::da::Fields* fields = new te::da::Fields();
  for (std::size_t i = 0; i < vecColumns.size(); ++i)
  {
    std::string columnName = vecColumns.at(i);
    fields->push_back(new te::da::Field(columnName));
  }

  te::da::From* from = new te::da::From();
  from->push_back(new te::da::DataSetName(tableName));

  te::da::Expression* whereExpression = nullptr;
  if (vecWhereColumns.empty() == false)
  {
    for (std::size_t i = 0; i < vecWhereColumns.size(); ++i)
    {
      std::string whereColumnName = vecWhereColumns.at(i);
      const te::dt::AbstractData* whereColumnValue = vecWhereValues.at(i).get();

      te::da::EqualTo* equal = new te::da::EqualTo(new te::da::PropertyName(whereColumnName), te::da::GetLiteralFromAbstractData(whereColumnValue));
      if (whereExpression == nullptr)
      {
        whereExpression = equal;

      }
      else
      {
        te::da::And* andExpression = new te::da::And(whereExpression, equal);
        whereExpression = andExpression;
      }
    }
  }

  std::unique_ptr<te::da::Select> selectPtr(new te::da::Select());
  if (distinct == true)
  {
    te::da::Distinct* distinctClause = new te::da::Distinct;
    selectPtr->setDistinct(distinctClause);
  }

  selectPtr->setFields(fields);
  selectPtr->setFrom(from);

  if (whereExpression != nullptr)
  {
    te::da::Where* where = new te::da::Where(whereExpression);
    selectPtr->setWhere(where);
  }

  return selectPtr;
}

std::unique_ptr<te::da::Select> te::da::BasicQueries::GetQuery(bool distinct, const std::vector<std::string>& vecColumns, const std::string& tableName)
{
  std::vector<std::string> vecWhereColumns;
  std::vector<std::unique_ptr<te::dt::AbstractData>> vecWhereValues;
  return GetQuery(distinct, vecColumns, tableName, vecWhereColumns, vecWhereValues);
}