/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralin/vp/AbstractDataHandler.h

   \brief This class represents handler to help reading and writing data from/to a dataSource by dividing it into smaller groups
 */

#ifndef __TERRALIB_VP_INTERNAL_BASICQUERIES_H
#define __TERRALIB_VP_INTERNAL_BASICQUERIES_H

#include "../Config.h"

#include <memory>
#include <string>
#include <vector>

namespace te
{
  namespace dt
  {
    class AbstractData;
  }

  namespace da
  {
    class Select;

    class BasicQueries
    {
      BasicQueries() = delete;
      ~BasicQueries() = delete;

    public:

      /*! \brief Helper function to create a query from the given parameters.
        \param distinct TRUE if distinct clause must be added. FALSE otherwise
        \param vecColumns A vector containing the columns to be added to the query
        \param tableName The name of the table to be queried
        \param vecWhereColumns A vector containing all the columns in the where clause
        \param vecWhereValues A vector containing all the values in the where clause
        \return An query resulting from the given parameters
    */
      TEDATAACCESSEXPORT static std::unique_ptr<te::da::Select> GetQuery(
        bool distinct,
        const std::vector<std::string>& vecColumns,
        const std::string& tableName,
        const std::vector<std::string>& vecWhereColumns,
        const std::vector<std::unique_ptr<te::dt::AbstractData>>& vecWhereValues);


      /*! \brief Helper function to create a query from the given parameters.
          \param distinct TRUE if distinct clause must be added. FALSE otherwise
          \param vecColumns A vector containing the columns to be added to the query
          \param tableName The name of the table to be queried
          \return An query resulting from the given parameters
      */
      TEDATAACCESSEXPORT static std::unique_ptr<te::da::Select> GetQuery(
        bool distinct,
        const std::vector<std::string>& vecColumns,
        const std::string& tableName);
    };
  }
}

#endif //__TERRALIB_VP_INTERNAL_BASICQUERIES_H
