/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/RasterEditWizardPage.h

  \brief This file defines a class for raster data edition.
*/

#ifndef __TERRALIB_QT_WIDGETS_RP_INTERNAL_RASTEREDITWIZARDPAGE_H
#define __TERRALIB_QT_WIDGETS_RP_INTERNAL_RASTEREDITWIZARDPAGE_H

// TerraLib
#include "../../../geometry/Envelope.h"
#ifndef Q_MOC_RUN
#include "../../../maptools/AbstractLayer.h"
#include "../../../qt/widgets/canvas/MapDisplay.h"
#endif
#include "../Config.h"

// STL
#include <memory>

// Qt
#include <QWizardPage>
#include <QActionGroup>

// Forward declaration
namespace Ui { class RasterEditWizardPageForm; }

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class RpToolsWidget;

      /*!
        \class RasterEditWizardPage

        \brief This class is GUI used to define the parameters raster data edition.
      */
      class TEQTWIDGETSEXPORT RasterEditWizardPage : public QWizardPage
      {
        Q_OBJECT

        public:

          enum RASTER_EDITION_TYPE
          {
            EDITION_ENTIRE_RASTER,
            EDITION_ROI_ASSISTED
          };

          RasterEditWizardPage(QWidget* parent = 0);

          ~RasterEditWizardPage();

          bool isComplete() const;

        public:

//           /*!
//             \brief This method is used to set the selected layer for clipping operation
//
//             \param layer The layer ptr
//
//             \note This layer MUST HAVE a valid raster object.
//           */
           void setLayer(te::map::AbstractLayerPtr layer);
//
//           void setList(std::list<te::map::AbstractLayerPtr>& layerList);
//
           void setMapDisplay(te::qt::widgets::MapDisplay* mapDisplay);
//
           void setActionGroup(QActionGroup* actionGroup);
//
           te::map::AbstractLayerPtr getLayer();
//
           RASTER_EDITION_TYPE getEditionType();

           int getBand();

           void getROIs(std::vector<te::gm::Geometry*> &geomVec);

           void clearCanvas();

           void drawGeom();

           void getReplaceValues( std::vector< std::pair< double, double > >& replaceValues ) const;

        public slots:

          void addValuesPushButtonClicked();

          void removeValuesPushButtonClicked();

          void onEnvelopeAcquired(te::gm::Envelope env);

          void onGeomAquired(te::gm::Polygon* poly);

          void onSelectRoiItem();

          void onRemoveRoiItem();

        protected:

        private:

          void addRoiItem();

        private:

          std::unique_ptr<Ui::RasterEditWizardPageForm> m_ui;
          std::unique_ptr<te::qt::widgets::RpToolsWidget> m_navigator;
          te::map::AbstractLayerPtr m_layer;
          std::vector<te::gm::Geometry*> m_geomVec;
          te::qt::widgets::MapDisplay* m_mapDisplay;
      };

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_RP_INTERNAL_RASTEREDITWIZARDPAGE_H
