/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/vp/SubdivideOperation.h

   \brief This operation subdivides geometries ensuring that they don't have more than the given number of coordinates
 */

#ifndef __TERRALIB_VP_INTERNAL_SUBDIVIDEOPERATION_H
#define __TERRALIB_VP_INTERNAL_SUBDIVIDEOPERATION_H

 //Terralib include files
#include "Config.h"

#include "AbstractOperation.h"

#include <mutex>
#include <vector>

namespace te
{
  namespace vp
  {
    class FeatureSet;

    class TEVPEXPORT SubdivideOperationParameters : public te::vp::AbstractParameters
    {
      public:
        SubdivideOperationParameters(std::size_t maxCoordinates = 0);
        ~SubdivideOperationParameters();

        void setMaxCoordinates(std::size_t maxCoordinates);

        std::size_t getMaxCoordinates() const;

    private:
      std::size_t m_maxCoordinates;
    };
    
    /*!
      \class SubdivideOperation
      \brief This operation subdivides geometries ensuring that they don't have more than the given number of coordinates
    */
    class TEVPEXPORT SubdivideOperation : public te::vp::AbstractOperation
    {
    public:

      /*!
        * \brief Constructor
        */
      SubdivideOperation();

      //!< Destructor
      virtual ~SubdivideOperation();

      virtual std::unique_ptr<te::vp::AbstractOperationCapabilities> getCapabilities() const override;

      std::vector<te::vp::FeatureSet> executeImpl(const std::vector<te::vp::FeatureSet>& vecInput) override;

      te::da::DataSetType* createBasicOutputDataSetType(const std::string& dataSetName, int srid, te::gm::GeomType geometryType, const std::string& geometryColumnName) const override;

    protected:

      std::size_t safeCreateParentId();

    protected:

      mutable std::size_t m_parentIdColumnIndex;
      std::size_t m_parentId;
      std::mutex m_mutex;
    };

    class TEVPEXPORT SubdivideOperationFactory : public te::vp::OperationFactory
    {
    public:

      SubdivideOperationFactory();

      virtual ~SubdivideOperationFactory();

    protected:

      virtual te::vp::AbstractOperation* build() override;
    };

  }//end namespace vp
}// end namespace te

#endif // __TERRALIB_VP_INTERNAL_MAKEVALIDOPERATION_H
